#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_embeddings
# @Time         : 2024/1/11 09:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.notice.feishu import send_message

from openai import OpenAI

send_message = partial(
    send_message,
    title="Embedding主备",
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e2f5c8eb-4421-4a0b-88ea-e2d9441990f2"
)


@lru_cache()
class Embeddings(object):
    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None, **kwargs):
        # 需要兜底的模型
        params = dict(
            api_key=api_key or 'sk-...',
            base_url=base_url or 'https://api.githubcopilot.com',
            default_headers={'Editor-Version': 'vscode/1.85.1'},
        )
        self.client = OpenAI(**params)  # todo: 异步

        self.backup_client = OpenAI(
            api_key=os.getenv("BACKUP_API_KEY"),
            base_url=os.getenv("OPENAI_BASE_URL", "https://api.chatllm.vip/v1")
        )

    def create(self, **data) -> Any:
        # data = {key: data.get(key) for key in completion_keys if key in data}  # 去除多余key
        try:
            response = self.client.embeddings.create(**data)
            return response

        except Exception as primary_error:  # 走兜底
            _ = f"Primary client failed: {primary_error}"
            send_message(_)
            logging.error(_)

            try:
                return self._backup_create(**data)
            except Exception as backup_error:
                _ = f"Backup client failed: {backup_error}"
                send_message(_)
                logging.error(_)
                return self._handle_error(data, backup_error)

    def _backup_create(self, **data):
        """恢复模型名"""
        backup_data = data.copy()
        backup_data['model'] = f"backup-{data['model']}"  # "backup-text-embedding-ada-002"
        backup_response = self.backup_client.embeddings.create(**backup_data)

        send_message(f"入参：{data}")  # 兜底监控

        return backup_response

    def _handle_error(self, data: Dict[str, Any], error: Exception) -> Any:
        """
        Handle errors and return an appropriate response.
        """
        return f"{data}: {error}"


if __name__ == '__main__':
    api_key = """
tid=925df46f15a3245bfa77d9f47cc073e1;exp=1704937860;sku=monthly_subscriber;st=dotcom;chat=1;rt=1;8kp=1:3102f85ef08442a7741d97ee67a403d6d61b62ba53f084a68171e00b325c804f    """.strip()
    print(Embeddings(api_key=api_key).create(input=['傻逼'] * 5, model='text-embedding-ada-002'))
